@@ -230,7 +230,7 @@ AddOn.prototype.preferenceFileExists = function() {
 
 /**
  * \return True if shortcuts can be defined for this add-on (default) or
- *      false if a add-on explicitely defines, that is does not support
+ *      false if a add-on explicitly defines, that is does not support
  *      shortcuts (ClassName.hasShortcuts()).
  */
 AddOn.prototype.hasShortcuts = function() {
@@ -457,17 +457,17 @@ AddOn.getAddOns = function(dir) {
     var fileMenuList, i, k;
 
     var addOns = [];
-    var dirFilter = new QDir.Filters(QDir.NoDotAndDotDot, QDir.Readable, QDir.Dirs);
-    var sortFlags = new QDir.SortFlags(QDir.NoSort);
+    var dirFilter = makeQDirFilters(QDir.NoDotAndDotDot, QDir.Readable, QDir.Dirs);
+    var sortFlags = makeQDirSortFlags(QDir.NoSort);
     
     // first call without recursion:
     if (topCall) {
-        var args = QCoreApplication.arguments();
+        var args = RSettings.getOriginalArguments();
 
         // ignore all scripts in ./scripts directory
         // and only load scripts from plugins
         // used mainly for testing / development
-        if (RSettings.getOriginalArguments().contains("-ignore-script-files")) {
+        if (args.contains("-ignore-script-files")) {
             dir = undefined;
         }
         else {
@@ -525,7 +525,12 @@ AddOn.getAddOns = function(dir) {
         var fileInfo = new QFileInfo(dirInfo.filePath() + "/" + dirInfo.fileName() + ".js");
 
         if (dirInfo.fileName().startsWith("AutoLoad")) {
-            RAutoLoadEcma.addAutoLoadFile(fileInfo.filePath());
+            if (typeof(RAutoLoadEcma)!=="undefined") {
+                RAutoLoadEcma.addAutoLoadFile(fileInfo.filePath());
+            }
+            if (typeof(RAutoLoadJs)!=="undefined") {
+                RAutoLoadJs.addAutoLoadFile(fileInfo.filePath());
+            }
         }
         else {
             if (fileInfo.exists() && !AddOn.isIgnored(fileInfo.absoluteFilePath())) {
@@ -647,14 +652,14 @@ AddOn.getAddOns = function(dir) {
 
 AddOn.getLocalAddOns = function() {
     var dataDir = new QDir(RSettings.getDataLocation());
-    var fs = new QDir.Filters(QDir.NoDotAndDotDot, QDir.Readable, QDir.Dirs, QDir.Executable);
-    var sf = new QDir.SortFlags(QDir.Name);
+    var fs = makeQDirFilters(QDir.NoDotAndDotDot, QDir.Readable, QDir.Dirs, QDir.Executable);
+    var sf = makeQDirSortFlags(QDir.Name);
     return dataDir.entryList([], fs, sf);
 };
 
 AddOn.isIgnored = function(path) {
     if (isNull(AddOn.ignores)) {
-        var args = QCoreApplication.arguments();
+        var args = RSettings.getOriginalArguments();
         AddOn.ignores = [];
         for (var i=1; i < args.length; ++i) {
             if (args[i] === "-ignore") {
