@@ -17,16 +17,10 @@
  * along with QCAD.
  */
 
-include("WidgetFactory.js");
+include("scripts/library.js");
+include("scripts/WidgetFactory.js");
 include("scripts/Widgets/OptionsToolBar/OptionsToolBar.js");
 
-if (RSettings.isGuiEnabled()) {
-    if (exists("scripts/Widgets/CadToolBar/CadToolBar.js")) {
-        include("scripts/Widgets/CadToolBar/ColumnLayout.js");
-        include("scripts/Widgets/CadToolBar/CadToolBar.js");
-    }
-}
-
 /**
  * Base class for all ECMAScript based actions.
  */
@@ -59,6 +53,8 @@ function EAction(guiAction) {
     this.waitingForContextMenu = false;
 
     this.optOutRelativeZeroResume = false;
+
+    this.resuming = false;
 }
 
 EAction.prototype = new RActionAdapter();
@@ -151,10 +147,10 @@ EAction.prototype.finishEvent = function() {
 
     // reset CAD tool bar:
     if (!isNull(guiAction) && guiAction.getGroup() === "") {
-        if (typeof(CadToolBar) !== "undefined") {
-            if (CadToolBar.getCurrentPanelName()==="SnapToolsPanel") {
-                CadToolBar.back();
-            }
+        var appWin = EAction.getMainWindow();
+        var cadToolBar = appWin.findChild("CadToolBar");
+        if (!isNull(cadToolBar) && cadToolBar.getCurrentPanelName()==="SnapToolsPanel") {
+            cadToolBar.back();
         }
     }
 
@@ -249,7 +245,7 @@ EAction.prototype.initState = function() {
 };
 
 /**
- * Called whenver the action resumes its operation, for example after it was
+ * Called whenever the action resumes its operation, for example after it was
  * temporary suspended for another action.
  */
 EAction.prototype.resumeEvent = function() {
@@ -283,7 +279,7 @@ EAction.prototype.resumeEvent = function() {
 };
 
 /**
- * Called whenver the action is suspended, for example if another action is
+ * Called whenever the action is suspended, for example if another action is
  * started while this action is active.
  */
 EAction.prototype.suspendEvent = function() {
@@ -356,6 +352,8 @@ EAction.prototype.showUiOptions = function(resume, restoreFromSettings) {
         return;
     }
 
+    this.resuming = true;
+
     this.optionWidgetActions = [];
     for (var i = 0; i < this.uiFile.length; ++i) {
         var uiFile = this.uiFile[i];
@@ -389,11 +387,11 @@ EAction.prototype.showUiOptions = function(resume, restoreFromSettings) {
         if (optionsToolBar.floating) {
             optionsToolBar.resize(optionsToolBar.sizeHint.width(), optionsToolBar.height);
         }
-        wOptions.destroy();
+        destr(wOptions);
 
         // automatically add separator to toolbar:
         var a = optionsToolBar.addSeparator();
-        a.objectName = "LastSeparator";
+        a.objectName = "LastSeparator" + i;
         this.optionWidgetActions.push(a);
 
         // give action a chance to initialize toolbar widgets that cannot
@@ -408,6 +406,8 @@ EAction.prototype.showUiOptions = function(resume, restoreFromSettings) {
 
     // hide options tool bar widgets which are shown in a dialog instead:
     this.hideOptionsToolBarWidgets();
+
+    this.resuming = false;
 };
 
 /**
@@ -424,6 +424,10 @@ EAction.prototype.initUiOptions = function(resume, optionsToolBar) {
     var children = optionsToolBar.children();
     for (var i=0; i<children.length; i++) {
         var child = children[i];
+        if (isNull(child)) {
+            continue;
+        }
+
         var shortCut = child.shortcut;
         if (!isNull(shortCut) && !shortCut.isEmpty()) {
             var str = shortCut.toString();
@@ -492,7 +496,7 @@ EAction.prototype.hideUiOptions = function(saveToSettings) {
         if (!isNull(optionsToolBar)) {
             optionsToolBar.removeAction(a);
         }
-        a.destroy();
+        destr(a);
     }
 
     // delete additional toolbars of this tool if available:
@@ -502,7 +506,7 @@ EAction.prototype.hideUiOptions = function(saveToSettings) {
             if (isDeleted(tb)) {
                 continue;
             }
-            tb.destroy();
+            destr(tb);
         }
     }
 
@@ -538,8 +542,12 @@ EAction.prototype.hideOptionsToolBarWidgets = function(widgets, noSyncWidgets) {
     var children = optionsToolBar.children();
     for (i = 0; i < children.length; ++i) {
         c = children[i];
+        if (isNull(c)) {
+            continue;
+        }
+
         if (c["HideInDialogMode"]===true) {
-            c.destroy();
+            destr(c);
         }
         else if (c["MoveToDialog"]===true) {
             widgets.push(c);
@@ -659,7 +667,7 @@ EAction.prototype.showDialog = function() {
         noSyncWidgets[i].setProperty("Loaded", false);
     }
 
-    this.dialog.destroy();
+    destr(this.dialog);
     this.dialog = undefined;
 
 //    var view = EAction.getGraphicsView();
@@ -749,18 +757,24 @@ EAction.prototype.keyReleaseEvent = function(event) {
  */
 EAction.prototype.setCrosshairCursor = function() {
     if (isNull(EAction.crossCursor)) {
-        var bitmap, mask
-        if (RSettings.getDevicePixelRatio()===2 && RS.getSystemId()!=="osx") {
-            bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor@2x.png", "PNG");
-            mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask@2x.png", "PNG");
-            //bitmap.setDevicePixelRatio(2);
-            //mask.setDevicePixelRatio(2);
-            EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
+
+        if (RSettings.getBoolValue("GraphicsView/SystemCursors", false)===true) {
+            EAction.crossCursor = new QCursor(Qt.CrossCursor);
         }
         else {
-            bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor.png", "PNG");
-            mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask.png", "PNG");
-            EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
+            var bitmap, mask
+            if (RSettings.getDevicePixelRatio()===2 && RS.getSystemId()!=="osx") {
+                bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor@2x.png", "PNG");
+                mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask@2x.png", "PNG");
+                //bitmap.setDevicePixelRatio(2);
+                //mask.setDevicePixelRatio(2);
+                EAction.crossCursor = new QCursor(bitmap, mask, 30, 30);
+            }
+            else {
+                bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor.png", "PNG");
+                mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask.png", "PNG");
+                EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
+            }
         }
     }
 
@@ -967,9 +981,10 @@ EAction.getMenu = function(title, objectName, initFunction) {
     var appWin = EAction.getMainWindow();
     var menuBar = appWin.menuBar();
     var menu;
-    for ( var i in menuBar.children()) {
-        menu = menuBar.children()[i];
-        if (menu.objectName === objectName) {
+    var children = menuBar.children();
+    for (var i in children) {
+        menu = children[i];
+        if (!isNull(menu) && menu.objectName === objectName) {
             break;
         }
         menu = undefined;
@@ -991,7 +1006,7 @@ EAction.getMenu = function(title, objectName, initFunction) {
  * icon.
  * 
  * \param menu Parent menu.
- * \param sortOrder Sort oder among other submenus. All
+ * \param sortOrder Sort order among other submenus. All
  *      submenus within the same menu are ordered by this number.
  * \param title Translated title of the submenu. E.g. qsTr("&Line").
  * \param objectName Object name to use for the menu. Used mainly for
@@ -1079,49 +1094,18 @@ EAction.getToolBar = function(title, objectName, toolBarArea, category, before)
         return undefined;
     }
 
-    var tb;
+    var tb = undefined;
     if (objectName.length !== 0) {
         tb = appWin.findChild(objectName);
     }
 
     if (!isNull(tb) && !isOfType(tb, QToolBar)) {
-        qWarning("Not a toolbar: ", tb);
+        qWarning("Not a toolbar: " + tb);
         return undefined;
     }
 
     if (isNull(tb)) {
         tb = new QToolBar(title);
-
-        // style tool buttons in options toolbar:
-        if (RSettings.isQt(5)) {
-            if (!RSettings.hasCustomStyleSheet()) {
-                if (RSettings.hasDarkGuiBackground()) {
-                    tb.setStyleSheet(
-                          "QToolButton {"
-                             // make sure unchecked button renders with same size as checked
-                             // prevents things from moving around when checking / unchecking
-                        + "  border: 1px solid transparent;"
-                        + "} "
-                        + "QToolButton:checked { "
-                        + "  border-top: 1px solid #161616;"
-                        + "  border-left: 1px solid #161616;"
-                        + "  border-bottom: 1px solid #777777;"
-                        + "  border-right: 1px solid #777777;"
-                        + "  background-color: #222222;"
-                        + "}");
-                }
-                else {
-                    tb.setStyleSheet(
-                        "QToolButton {" +
-                        "  border: 1px solid transparent;" +
-                        "} " +
-                        "QToolButton:checked { " +
-                        "  border:1px solid #7f7f7f; " +
-                        "  background: qlineargradient(x1:0 y1:0, x2:0 y2:1 stop:0 #c0c0c0, stop:0.1 #8a8a8a stop:0.2 #a3a3a3 stop:1 transparent); " +
-                        "}");
-                }
-            }
-        }
         tb.objectName = objectName;
         if (!isNull(category)) {
             tb.setProperty("Category", category);
@@ -1171,7 +1155,7 @@ EAction.getOptionsToolBar = function() {
     }
 
     if (!isNull(EAction.optionsToolBar)) {
-        if (!QCoreApplication.arguments().contains("-no-show")) {
+        if (!RSettings.getOriginalArguments().contains("-no-show")) {
             EAction.optionsToolBar.visible = true;
         }
     }
@@ -1211,46 +1195,37 @@ EAction.getMainCadToolBarPanel = function() {
  * back to the main CAD toolbar panel.
  */
 EAction.getCadToolBarPanel = function(title, objectName, hasBackButton) {
-    if (typeof(CadToolBar) === "undefined") {
-        return undefined;
-    }
-
     var appWin = EAction.getMainWindow();
-    var cadToolBar = appWin.findChild("CadQToolBar");
+    var cadToolBar = appWin.findChild("CadToolBar");
 
     // create CAD toolbar if it does not exist already:
     if (isNull(cadToolBar)) {
-        var toolBar = new QToolBar(qsTr("CAD Tools"), appWin);
-        toolBar.objectName = "CadQToolBar";
-        cadToolBar = new CadToolBar(toolBar);
-        toolBar.addWidget(cadToolBar);
-        cadToolBar.updateIconSize();
+        cadToolBar = new RCadToolBar(qsTr("CAD Tools"), appWin);
+        cadToolBar.objectName = "CadToolBar";
+//        cadToolBar.updateIconSize();
 
-        toolBar.topLevelChanged.connect(function(onOff) {
+        cadToolBar.topLevelChanged.connect(function(onOff) {
             RSettings.setValue("CadToolBar/VerticalWhenFloating", false);
         });
-        toolBar.orientationChanged.connect(function(orientation) {
+        cadToolBar.orientationChanged.connect(function(orientation) {
             RSettings.setValue("CadToolBar/VerticalWhenFloating", false);
         });
     }
 
-    return CadToolBar.getPanel(title, objectName, hasBackButton);
+    return cadToolBar.getPanel(title, objectName, hasBackButton);
 };
 
 /**
  * Shows the CAD toolbar panel with the given objectName.
  */
 EAction.showCadToolBarPanel = function(objectName) {
-    if (typeof (CadToolBar) == "undefined") {
-        return;
-    }
-
     var appWin = EAction.getMainWindow();
     if (isNull(appWin)) {
         return;
     }
 
-    CadToolBar.showPanel(objectName);
+    var toolBar = appWin.findChild("CadToolBar");
+    toolBar.showPanel(objectName);
 };
 
 /**
@@ -1379,6 +1354,8 @@ EAction.addGuiActionTo = function(action, iface, addToMenu, addToToolBar,
     if (isNull(addSeparator)) {
         addSeparator = false;
     }
+    var addToUserToolBar1 = false;
+    var addToUserToolBar2 = false;
 
     if (className.length!==0) {
         var key = className + "/VisibleInMenu";
@@ -1396,14 +1373,22 @@ EAction.addGuiActionTo = function(action, iface, addToMenu, addToToolBar,
         if (!RSettings.hasValue(key)) {
             RSettings.setValue(key, addToCadToolBar);
         }
+        key = className + "/VisibleInUserToolBar1";
+        addToUserToolBar1 = RSettings.getBoolValue(key, addToUserToolBar1);
+        if (!RSettings.hasValue(key)) {
+            RSettings.setValue(key, addToUserToolBar1);
+        }
+        key = className + "/VisibleInUserToolBar2";
+        addToUserToolBar2 = RSettings.getBoolValue(key, addToUserToolBar2);
+        if (!RSettings.hasValue(key)) {
+            RSettings.setValue(key, addToUserToolBar2);
+        }
     }
 
     if (isNull(iface)) {
         qWarning("EAction.js:", "addGuiActionTo(): iface not defined");
     }
 
-
-
     if (action.icon.isNull() && !action.isIconDisabled()) {
         action.setIcon(autoPath("scripts/Empty.svg"));
     }
@@ -1455,6 +1440,21 @@ EAction.addGuiActionTo = function(action, iface, addToMenu, addToToolBar,
             }
         }
     }
+
+    var appWin = EAction.getMainWindow();
+    if (addToUserToolBar1) {
+        var tb = appWin.findChild("UserToolBar1");
+        if (!isNull(tb)) {
+            action.addToToolBar(tb);
+        }
+    }
+
+    if (addToUserToolBar2) {
+        var tb = appWin.findChild("UserToolBar2");
+        if (!isNull(tb)) {
+            action.addToToolBar(tb);
+        }
+    }
 };
 
 //EAction.addGuiActionToWidgets = function(action, widgetNames) {
@@ -1638,8 +1638,16 @@ EAction.getScales = function(unit) {
                 '1" = 512"', '1" = 1024"', '1" = 2048"', '1" = 4096"',
                 '1/4096" = 1\'-0"', '1/2048" = 1\'-0"', '1/1024" = 1\'-0"',
                 '1/512" = 1\'-0"', '1/256" = 1\'-0"', '1/128" = 1\'-0"',
-                '1/64" = 1\'-0"', '1/32" = 1\'-0"', '1/16" = 1\'-0"',
-                '1/8" = 1\'-0"', '1/4" = 1\'-0"', '3/4" = 1\'-0"', '1/2" = 1\'-0"',
+                '1/64" = 1\'-0"', 
+                '1/32" = 1\'-0"', 
+                '1/16" = 1\'-0"',
+                '3/32" = 1\'-0"', 
+                '1/8" = 1\'-0"', 
+                '3/16" = 1\'-0"', 
+                '1/4" = 1\'-0"', 
+                '3/8" = 1\'-0"',
+                '1/2" = 1\'-0"',
+                '3/4" = 1\'-0"', 
                 '1" = 1\'-0"', '2" = 1\'-0"', '3" = 1\'-0"', '4" = 1\'-0"',
                 '5" = 1\'-0"', '6" = 1\'-0"', '7" = 1\'-0"', '8" = 1\'-0"',
                 '9" = 1\'-0"', '10" = 1\'-0"', '11" = 1\'-0"');
@@ -1664,7 +1672,7 @@ EAction.prototype.getAuxPreview = function() {
 };
 
 /**
- * Called by updatePreview. Implementations must return an array of enitity IDs
+ * Called by updatePreview. Implementations must return an array of entity IDs
  * which should be highlighted.
  */
 EAction.prototype.getHighlightedEntities = function() {
@@ -1736,7 +1744,8 @@ EAction.prototype.applyOperation = function() {
         return false;
     }
     this.complementOperation(op);
-    di.applyOperation(op);
+    var t = di.applyOperation(op);
+    this.postOperation(t);
     return true;
 };
 
@@ -1747,6 +1756,13 @@ EAction.prototype.complementOperation = function(op) {
     return;
 };
 
+/**
+ * Can be reimplemented to work with the transaction returned by the operation.
+ */
+EAction.prototype.postOperation = function(transaction) {
+    return;
+};
+
 /**
  * High level event triggered when a coordinate was specified by the user of a
  * coordinate specification should be previewed.
@@ -1758,6 +1774,13 @@ EAction.prototype.pickCoordinate = function(event, preview) {
  * Default implementation. Calls pickCoordinate(event, false)
  */
 EAction.prototype.coordinateEvent = function(event) {
+    // don't let invalid positions go through for most cases (this is overridden for DefaultAction):
+    var pos = event.getModelPosition();
+    if (!isValidVector(pos)) {
+        EAction.handleUserWarning(qsTr("Invalid position"));
+        return;
+    }
+
     this.pickCoordinate(event, false);
 };
 
@@ -1930,7 +1953,12 @@ EAction.setProgressEnd = function() {
 EAction.assertEditable = function(entity, quiet) {
     if (!entity.isEditable()) {
         if (!quiet) {
-            EAction.handleUserWarning(qsTr("Entity is on a locked layer."));
+            if (!entity.isInWorkingSet()) {
+                EAction.handleUserWarning(qsTr("Entity is not in working set."));
+            }
+            else {
+                EAction.handleUserWarning(qsTr("Entity is on a locked layer."));
+            }
         }
         return false;
     }
@@ -1939,7 +1967,7 @@ EAction.assertEditable = function(entity, quiet) {
 };
 
 /**
- * Allows all actions to handle property change events. This is neccessary to
+ * Allows all actions to handle property change events. This is necessary to
  * ensure that properties can be changed even if a tool is active (e.g. while
  * drawing lines).
  */
@@ -2174,8 +2202,8 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
         }
 
         reactor = new Reactor(id);
-        a.triggered.connect(reactor, "trigger");
-        a.hovered.connect(reactor, "hover");
+        a.triggered.connect(reactor, reactor.trigger);
+        a.hovered.connect(reactor, reactor.hover);
     }
 
     // show context menu:
@@ -2192,6 +2220,28 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
     return ret;
 };
 
+EAction.prototype.isEntitySnappable = function(e) {
+    if (!isEntity(e)) {
+        return false;
+    }
+
+    var layerId = e.getLayerId();
+    var data = e.getData();
+    var doc = data.getDocument();
+    if (isNull(doc)) {
+        return false;
+    }
+
+    return doc.isLayerSnappable(layerId);
+};
+
+EAction.prototype.isEntityEditable = function(e) {
+    if (!isEntity(e)) {
+        return false;
+    }
+    return e.isEditable();
+};
+
 /**
  * Some common, shared translated warnings:
  */
@@ -2223,6 +2273,10 @@ EAction.warnNotLineArcCircle = function() {
     EAction.handleUserWarning(qsTr("Entity is not a line, arc or circle."));
 };
 
+EAction.warnNotLineArcSplineEllipseArcPolyline = function() {
+    EAction.handleUserWarning(qsTr("Entity is not a line, arc, spline, ellipse arc or polyline."));
+};
+
 EAction.warnNotArcCircleEllipse = function() {
     EAction.handleUserWarning(qsTr("Entity is not an arc, circle or ellipse."));
 };
@@ -2250,3 +2304,7 @@ EAction.warnNotLineArcCircleEllipseSplinePolyline = function() {
 EAction.warnNotPolyline = function() {
     EAction.handleUserWarning(qsTr("Entity is not a polyline."));
 };
+
+EAction.warnNotSplineWithFitPoints = function() {
+    EAction.handleUserWarning(qsTr("Entity is not a spline with fit points."));
+};
