@@ -19,7 +19,7 @@
 
 include("scripts/EAction.js");
 include("scripts/sprintf.js");
-include("../../WidgetFactory.js");
+include("scripts/WidgetFactory.js");
 
 /**
  * Internal helper class. Notified when properties are changed.
@@ -45,7 +45,7 @@ PropertyWatcher.prototype.propertyChanged = function(value) {
 
     var attributes = this.propertyEditor.getPropertyAttributes(this.propertyType);
 
-    var typeHint = 0;
+    var typeHint = RS.UnknownType;
 
     // value is a list property (e.g. x coordinate of a vertex of a polyline):
     if (attributes.isList()) {
@@ -79,7 +79,7 @@ PropertyWatcher.prototype.propertyChanged = function(value) {
     }
 
     // value is string from a line edit (e.g. text contents):
-    else if (this.sender.toString()==="QLineEdit") {
+    else if (isOfType(this.sender, QLineEdit)) {
         value = this.sender.text;
         if (value===PropertyEditor.varies) {
             return;
@@ -87,7 +87,7 @@ PropertyWatcher.prototype.propertyChanged = function(value) {
     }
 
     // value is number from a math line edit:
-    else if (this.sender.toString().startsWith("RMathLineEdit")) {
+    else if (isOfType(this.sender, RMathLineEdit)) {
         if (this.sender.text===this.sender.originalText) {
             return;
         }
@@ -98,12 +98,12 @@ PropertyWatcher.prototype.propertyChanged = function(value) {
             return;
         }
         if (this.sender.isInteger()) {
-            typeHint = 2;
+            typeHint = RS.Int;
         }
     }
 
     // value is number from a math combo box:
-    else if (this.sender.toString().startsWith("RMathComboBox")) {
+    else if (isOfType(this.sender, RMathComboBox)) {
         if (this.sender.currentText===this.sender.originalText) {
             return;
         }
@@ -124,7 +124,13 @@ PropertyWatcher.prototype.propertyChanged = function(value) {
  * that match the current entity type filter.
  */
 PropertyWatcher.prototype.propertyRemoved = function() {
-    this.propertyEditor.propertyChanged(this.propertyType, null);
+    if (RSettings.getQtVersion() >= 0x060000) {
+        this.propertyEditor.propertyChanged(this.propertyType, new QVariant());
+    }
+    else {
+        this.propertyEditor.propertyChanged(this.propertyType, null);
+    }
+
     this.propertyEditor.onlyChangesOverride = false;
 };
 
@@ -222,36 +228,61 @@ function PropertyEditorImpl(basePath) {
     this.widget.findChild("LabelProtected").text = RSettings.translate("REntity", "Protected") + this.colon;
 
     var selectionCombo = this.widget.findChild("Selection");
-    selectionCombo["activated(int)"].connect(this, "filterChanged");
+    selectionCombo["activated(int)"].connect(this, this.filterChanged);
     selectionCombo.installEventFilter(new REventFilter(QEvent.Wheel.valueOf(), true));
     selectionCombo.focusPolicy = Qt.ClickFocus;
 
     // initialize fixed general properties at the top:
     var layerCombo = this.widget.findChild("Layer");
-    layerCombo['activated(QString)'].connect(
-                new PropertyWatcher(this, layerCombo, REntity.PropertyLayer),
-                'propertyChanged');
+    if (RSettings.isQt(6)) {
+        var pw = new PropertyWatcher(this, layerCombo, REntity.PropertyLayer);
+        layerCombo.textActivated.connect(pw, pw.propertyChanged);
+    }
+    else {
+        layerCombo['activated(QString)'].connect(
+                    new PropertyWatcher(this, layerCombo, REntity.PropertyLayer),
+                    'propertyChanged');
+    }
+
     layerCombo.installEventFilter(new REventFilter(QEvent.Wheel.valueOf(), true));
     layerCombo.focusPolicy = Qt.ClickFocus;
 
     var colorCombo = this.widget.findChild("Color");
-    colorCombo['activated(int)'].connect(
-                new PropertyWatcher(this, colorCombo, REntity.PropertyColor),
-                'propertyChanged');
+    if (RSettings.isQt(6)) {
+        var pw = new PropertyWatcher(this, colorCombo, REntity.PropertyColor);
+        colorCombo['activated(int)'].connect(pw, pw.propertyChanged);
+    }
+    else {
+        colorCombo['activated(int)'].connect(
+                    new PropertyWatcher(this, colorCombo, REntity.PropertyColor),
+                    'propertyChanged');
+    }
     colorCombo.installEventFilter(new REventFilter(QEvent.Wheel.valueOf(), true));
     colorCombo.focusPolicy = Qt.ClickFocus;
 
     var lineweightCombo = this.widget.findChild("Lineweight");
-    lineweightCombo['activated(int)'].connect(
-                new PropertyWatcher(this, lineweightCombo, REntity.PropertyLineweight),
-                'propertyChanged');
+    if (RSettings.isQt(6)) {
+        var pw = new PropertyWatcher(this, lineweightCombo, REntity.PropertyLineweight);
+        lineweightCombo['activated(int)'].connect(pw, pw.propertyChanged);
+    }
+    else {
+        lineweightCombo['activated(int)'].connect(
+                    new PropertyWatcher(this, lineweightCombo, REntity.PropertyLineweight),
+                    'propertyChanged');
+    }
     lineweightCombo.installEventFilter(new REventFilter(QEvent.Wheel.valueOf(), true));
     lineweightCombo.focusPolicy = Qt.ClickFocus;
 
     var linetypeCombo = this.widget.findChild("Linetype");
-    linetypeCombo['activated(int)'].connect(
-                new PropertyWatcher(this, linetypeCombo, REntity.PropertyLinetype),
-                'propertyChanged');
+    if (RSettings.isQt(6)) {
+        var pw = new PropertyWatcher(this, linetypeCombo, REntity.PropertyLinetype);
+        linetypeCombo['activated(int)'].connect(pw, pw.propertyChanged);
+    }
+    else {
+        linetypeCombo['activated(int)'].connect(
+                    new PropertyWatcher(this, linetypeCombo, REntity.PropertyLinetype),
+                    'propertyChanged');
+    }
     linetypeCombo.installEventFilter(new REventFilter(QEvent.Wheel.valueOf(), true));
     linetypeCombo.focusPolicy = Qt.ClickFocus;
 
@@ -273,14 +304,27 @@ function PropertyEditorImpl(basePath) {
 
 
     var linetypeScaleEdit = this.widget.findChild("LinetypeScale");
-    linetypeScaleEdit.editingFinished.connect(
-                new PropertyWatcher(this, linetypeScaleEdit, REntity.PropertyLinetypeScale),
-                'propertyChanged');
+    if (RSettings.isQt(6)) {
+        var pw = new PropertyWatcher(this, linetypeScaleEdit, REntity.PropertyLinetypeScale);
+        linetypeScaleEdit.editingFinished.connect(pw, pw.propertyChanged);
+    }
+    else {
+        linetypeScaleEdit.editingFinished.connect(
+                    new PropertyWatcher(this, linetypeScaleEdit, REntity.PropertyLinetypeScale),
+                    'propertyChanged');
+    }
+
 
     var drawOrderEdit = this.widget.findChild("DrawOrder");
-    drawOrderEdit.editingFinished.connect(
-                new PropertyWatcher(this, drawOrderEdit, REntity.PropertyDrawOrder),
-                'propertyChanged');
+    if (RSettings.isQt(6)) {
+        var pw = new PropertyWatcher(this, drawOrderEdit, REntity.PropertyDrawOrder);
+        drawOrderEdit.editingFinished.connect(pw, pw.propertyChanged);
+    }
+    else {
+        drawOrderEdit.editingFinished.connect(
+                    new PropertyWatcher(this, drawOrderEdit, REntity.PropertyDrawOrder),
+                    'propertyChanged');
+    }
 
     this.geometryGroup = undefined;
     this.childGroup = undefined;
@@ -312,15 +356,15 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
     this.widget.updatesEnabled = false;
     if (!onlyChanges) {
         if (!isNull(this.geometryGroup)) {
-            this.geometryGroup.destroy();
+            destr(this.geometryGroup);
             this.geometryGroup = undefined;
         }
         if (!isNull(this.childGroup)) {
-            this.childGroup.destroy();
+            destr(this.childGroup);
             this.childGroup = undefined;
         }
         if (!isNull(this.customGroup)) {
-            this.customGroup.destroy();
+            destr(this.customGroup);
             this.customGroup = undefined;
         }
     }
@@ -419,6 +463,7 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
         if (RSettings.isXDataEnabled()) {
             // create custom property group box with grid layout:
             this.customGroup = new QGroupBox(qsTr("Custom"), this.widget);
+            //this.customGroup.objectName = "CustomGroup";
             layout.insertWidget(4, this.customGroup);
 
             // grid layout with four columns and N rows for N property controls:
@@ -453,11 +498,16 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
             var children = groupToClear.children();
             for (var i=0; i<children.length; i++) {
                 var child = children[i];
+                if (isNull(child)) {
+                    // don't destroy wrapper attached to the group box (QGroupBox_Wrapper):
+                    continue;
+                }
                 if (isOfType(child, QGridLayout)) {
                     // don't destroy layout:
                     continue;
                 }
-                children[i].destroy();
+
+                destr(child);
             }
         }
     }
@@ -699,9 +749,8 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
                             removeCustomPropertyButton.objectName = "DeleteCustomProperty" + name;
                             //qDebug("adding button to remove custom property named: ", name);
                             var propertyEditor = this;
-                            removeCustomPropertyButton.clicked.connect(
-                                        new PropertyWatcher(this, removeCustomPropertyButton, propertyTypeId),
-                                        'propertyRemoved');
+                            var pw = new PropertyWatcher(this, removeCustomPropertyButton, propertyTypeId);
+                            removeCustomPropertyButton.clicked.connect(pw, pw.propertyRemoved);
                             gridLayoutCustom.addWidget(removeCustomPropertyButton, row,3);
                         }
 
@@ -789,7 +838,7 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
             addCustomPropertyButton.iconSize = new QSize(12,12);
             addCustomPropertyButton.toolTip = qsTr("Add custom property to selected objects");
             addCustomPropertyButton.objectName = "AddCustomProperty";
-            addCustomPropertyButton.clicked.connect(this, "addCustomProperty");
+            addCustomPropertyButton.clicked.connect(this, this.addCustomProperty);
             gridLayoutCustom.addWidget(addCustomPropertyButton, gridLayoutCustom.rowCount(),3, 1,1);
         }
     }
@@ -834,7 +883,7 @@ PropertyEditorImpl.prototype.initControls = function(propertyTypeId, onlyChanges
         requestAllButton.objectName = objectName + "_request";
         requestAllButton.text = qsTr("Show");
         requestAllButton.toolTip = qsTr("Show all properties");
-        requestAllButton.clicked.connect(this, "requestAllProperties");
+        requestAllButton.clicked.connect(this, this.requestAllProperties);
         requestAllButton.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Preferred);
 
         return new Array(requestAllButton);
@@ -936,7 +985,8 @@ PropertyEditorImpl.prototype.initControls = function(propertyTypeId, onlyChanges
                 }
                 indexControl.setProperty('controlNames', a);
 
-                indexControl["valueChanged(int)"].connect(new IndexWatcher(this, indexControl, propertyTypeId), "indexChanged");
+                var iw = new IndexWatcher(this, indexControl, propertyTypeId)
+                indexControl["valueChanged(int)"].connect(iw, iw.indexChanged);
             }
             else {
                 // make sure that the values for the current list index are updated:
@@ -1111,9 +1161,9 @@ PropertyEditorImpl.prototype.initNumberControls = function(objectName, propertyT
         else {
             lineEdit = control;
         }
-        lineEdit.editingFinished.connect(
-                    new PropertyWatcher(this, control, propertyTypeId),
-                    'propertyChanged');
+
+        var pw = new PropertyWatcher(this, control, propertyTypeId);
+        lineEdit.editingFinished.connect(pw, pw.propertyChanged);
     }
 
     // move cursor to start if field is not being edited:
@@ -1172,9 +1222,8 @@ PropertyEditorImpl.prototype.initStringControls = function(objectName, propertyT
     }
 
     if (!onlyChanges) {
-        control.editingFinished.connect(
-                    new PropertyWatcher(this, control, propertyTypeId),
-                    'propertyChanged');
+        var pw = new PropertyWatcher(this, control, propertyTypeId);
+        control.editingFinished.connect(pw, pw.propertyChanged);
     }
 
     // move cursor to start if field is not being edited:
@@ -1233,9 +1282,8 @@ PropertyEditorImpl.prototype.initBooleanControls = function(objectName, property
     }
 
     if (!onlyChanges) {
-        control['activated(int)'].connect(
-                    new PropertyWatcher(this, control, propertyTypeId),
-                    'propertyChanged');
+        var pw = new PropertyWatcher(this, control, propertyTypeId);
+        control['activated(int)'].connect(pw, pw.propertyChanged);
     }
 
     return new Array(control);
@@ -1247,7 +1295,7 @@ PropertyEditorImpl.prototype.initBooleanControls = function(objectName, property
  */
 PropertyEditorImpl.prototype.initChoiceControls = function(
     objectName, propertyTypeId, onlyChanges, control, choices, choicesData) {
-    
+
     var i;
 
     var value = this.getAdjustedPropertyValue(propertyTypeId);
@@ -1277,14 +1325,22 @@ PropertyEditorImpl.prototype.initChoiceControls = function(
         control.objectName = objectName;
 
         if (isNull(choicesData)/* && propertyTypeId.getId()!==REntity.PropertyLayer.getId()*/) {
-            control['activated(QString)'].connect(
-                        new PropertyWatcher(this, control, propertyTypeId),
-                        'propertyChanged');
+            var pw = new PropertyWatcher(this, control, propertyTypeId)
+            if (RSettings.getQtVersion() >= 0x060000) {
+                control.textActivated.connect(pw, pw.propertyChanged);
+            }
+            else {
+                control['activated(QString)'].connect(pw, pw.propertyChanged);
+            }
         }
         else {
-            control['activated(int)'].connect(
-                        new PropertyWatcher(this, control, propertyTypeId),
-                        'propertyChanged');
+            var pw = new PropertyWatcher(this, control, propertyTypeId);
+            if (RSettings.getQtVersion() >= 0x060000) {
+                control.activated.connect(pw, pw.propertyChanged);
+            }
+            else {
+                control['activated(int)'].connect(pw, pw.propertyChanged);
+            }
         }
     }
 
@@ -1498,12 +1554,12 @@ PropertyEditorImpl.prototype.makeReadOnly = function(control) {
     }
 
     var p = control.palette;
-    if (isNull(control.oriPalette)) {
+    if (isNull(control.property("oriPalette"))) {
         control.setProperty("oriPalette", control.palette);
     }
 
-    p.setColor(QPalette.Active, QPalette.Text, control.oriPalette.color(QPalette.Disabled, QPalette.WindowText));
-    p.setColor(QPalette.Inactive, QPalette.Text, control.oriPalette.color(QPalette.Disabled, QPalette.WindowText));
+    p.setColor(QPalette.Active, QPalette.Text, control.property("oriPalette").color(QPalette.Disabled, QPalette.WindowText));
+    p.setColor(QPalette.Inactive, QPalette.Text, control.property("oriPalette").color(QPalette.Disabled, QPalette.WindowText));
 
 //    if (RSettings.hasDarkGuiBackground()) {
 //        p.setColor(QPalette.Base, new QColor("#0a0a0a"));
@@ -1527,12 +1583,12 @@ PropertyEditorImpl.prototype.makeReadWrite = function(control) {
     }
 
     var p = control.palette;
-    if (isNull(control.oriPalette)) {
+    if (isNull(control.property("oriPalette"))) {
         control.setProperty("oriPalette", control.palette);
     }
 
-    p.setColor(QPalette.Active, QPalette.Text, control.oriPalette.color(QPalette.Active, QPalette.WindowText));
-    p.setColor(QPalette.Inactive, QPalette.Text, control.oriPalette.color(QPalette.Inactive, QPalette.WindowText));
+    p.setColor(QPalette.Active, QPalette.Text, control.property("oriPalette").color(QPalette.Active, QPalette.WindowText));
+    p.setColor(QPalette.Inactive, QPalette.Text, control.property("oriPalette").color(QPalette.Inactive, QPalette.WindowText));
 //    //if (RSettings.hasDarkGuiBackground()) {
 //        p.setColor(QPalette.Base, control.oriBase);
 ////    }
@@ -1564,7 +1620,7 @@ PropertyEditorImpl.prototype.addCustomProperty = function() {
     var valueEdit = dialog.findChild("Value");
 
     if (!dialog.exec()) {
-        dialog.destroy();
+        destr(dialog);
         EAction.activateMainWindow();
         return;
     }
@@ -1576,7 +1632,7 @@ PropertyEditorImpl.prototype.addCustomProperty = function() {
     this.propertyChanged(new RPropertyTypeId(RSettings.getAppId(), name), value, this.getEntityTypeFilter());
     this.onlyChangesOverride = false;
 
-    dialog.destroy();
+    destr(dialog);
     EAction.activateMainWindow();
 };
 
@@ -1607,7 +1663,7 @@ PropertyEditor.prototype.beginEvent = function() {
 
     var appWin = RMainWindowQt.getMainWindow();
     var dock = appWin.findChild("PropertyEditorDock");
-    if (!QCoreApplication.arguments().contains("-no-show")) {
+    if (!RSettings.getOriginalArguments().contains("-no-show")) {
         dock.visible = !dock.visible;
         if (dock.visible) dock.raise();
     }
@@ -1663,7 +1719,12 @@ PropertyEditor.init = function(basePath) {
     dock.objectName = "PropertyEditorDock";
     dock.setWidget(pe.widget);
     appWin.addPropertyListener(pe);
-    appWin.addLayerListener(pe.getRLayerListener());
+    if (RSettings.getQtVersion() >= 0x060000) {
+        appWin.addLayerListener(pe);
+    }
+    else {
+        appWin.addLayerListener(pe.getRLayerListener());
+    }
     appWin.addDockWidget(Qt.RightDockWidgetArea, dock);
     PropertyEditor.instance = pe;
 
