@@ -23,6 +23,15 @@ include("scripts/File/OpenFile/OpenFile.js");
 include("scripts/File/AutoSave/AutoSave.js");
 include("scripts/Tools/arguments.js");
 
+// allow plugins to extend autostart through autostart1.js, autostart2.js, ...:
+for (var i=1; i<10; i++) {
+    var fn = "scripts/autostart" + i + ".js";
+    if (new QFileInfo(fn).exists() || new QFileInfo(":/" + fn).exists()) {
+        include(fn);
+    }
+}
+
+
 /**
  * Prints version information.
  */
@@ -38,7 +47,7 @@ function version() {
  * Prints command line usage information on stdout.
  */
 function usage() {
-    print("\nUsage: " + QCoreApplication.arguments()[0] + " [Options] [Files to open]\n"
+    print("\nUsage: " + RSettings.getOriginalArguments()[0] + " [Options] [Files to open]\n"
           + "\n"
           + "-allow-multiple-instances        Don't try to avoid multiple instances from running\n"
           + "                                 simultaneously.\n"
@@ -63,8 +72,8 @@ function usage() {
           + "                                 after staring QCAD. Options after the script\n"
           + "                                 file are passed on to the script.\n"
           + "-help                            Displays this help.\n"
-          + "-ignore-script-files             Ignore script files on disk."
-          + "                                 Only load scripts from plugins if applicable."
+          + "-ignore-script-files             Ignore script files on disk.\n"
+          + "                                 Only load scripts from plugins if applicable.\n"
           + "-locale [locale]                 Sets the locale to be used (overrides\n"
           + "                                 the language set in the preferences).\n"
           + "                                 E.g. '-locale de' starts QCAD in German.\n"
@@ -271,7 +280,7 @@ function execScripts(args) {
 }
 
 /**
- * Sets up drag and drop support (droping files on the application window
+ * Sets up drag and drop support (dropping files on the application window
  * opens them).
  */
 function setUpDragAndDrop(appWin) {
@@ -311,7 +320,7 @@ function loadTranslations(addOns, splash) {
 
     // load C++ translations:
     var modules = ["qt", "assistant", "qt_help", "qcadcore", "qcadentity", "qcadgui"];
-    if (RSettings.isQt(5)) {
+    if (RSettings.isQt(5) || RSettings.isQt(6)) {
         modules.unshift("qtbase");
     }
 
@@ -327,7 +336,7 @@ function loadTranslations(addOns, splash) {
     //RSettings.loadTranslations("scripts_" + locale, [autoPath("scripts/ts")]);
 
     // load translations from arguments:
-    var args = QCoreApplication.arguments();
+    var args = RSettings.getOriginalArguments();
     for (i = 0; i < args.length; ++i) {
         if (args[i] === "-ts") {
             if (++i>=args.length) {
@@ -459,7 +468,7 @@ function main() {
     var i;
     var filesToOpen = [];
 
-    // open file that was clicked in Finder if application was not yet running (Mac OS X):
+    // open file that was clicked in Finder if application was not yet running (macOS):
     qApp.fileOpenRequestReceived.connect(function(fileName) {
         filesToOpen.push(fileName);
     });
@@ -472,7 +481,7 @@ function main() {
         execScripts(args);
     });
 
-    var args = QCoreApplication.arguments();
+    var args = RSettings.getOriginalArguments();
     if (args.contains("-help") || args.contains("-h")) {
         usage();
         return;
@@ -553,7 +562,7 @@ function main() {
     }
 
     // detect very first start of this installation:
-    var fiSettings = new QFileInfo(RSettings.getQSettings().fileName());
+    var fiSettings = new QFileInfo(RSettings.getFileName());
     var isFirstStart = !fiSettings.exists();
 
     var numPlugins = RPluginLoader.countPlugins();
@@ -562,15 +571,15 @@ function main() {
     // look up app name override:
     for (i=0; i<numPlugins; i++) {
         pluginInfo = RPluginLoader.getPluginInfo(i);
-        var n = pluginInfo.get("NameOverride");
-        if (!isNull(n)) {
+        var n = pluginInfo.get("NameOverride", "");
+        if (n.length>0) {
             qApp.applicationName = n;
         }
     }
 
     // if locale is given, don't show first start dialog:
-    if (isFirstStart && !QCoreApplication.arguments().contains("-locale")) {
-        include("Widgets/FirstStart/FirstStart.js");
+    if (isFirstStart && !args.contains("-locale")) {
+        include("scripts/Widgets/FirstStart/FirstStart.js");
         var first = new FirstStart();
         first.showDialog();
     }
@@ -640,7 +649,7 @@ function main() {
         var pixmap = new QPixmap(fn);
         splash = new QSplashScreen(pixmap);
         splash.objectName = "Splash";
-        if (!QCoreApplication.arguments().contains("-no-show")) {
+        if (!args.contains("-no-show")) {
             splash.show();
         }
     }
@@ -664,8 +673,8 @@ function main() {
     var addOns;
     var addOnFilePaths = RSettings.getValue("AddOns/List", []);
     if (addOnFilePaths.length===0 || newVersion ||
-            QCoreApplication.arguments().contains("-rescan") ||
-            RSettings.getBoolValue("Scripting/Rescan", true)===true) {
+        args.contains("-rescan") ||
+        RSettings.getBoolValue("Scripting/Rescan", true)===true) {
 
         if (!isNull(splash)) {
             // no translations yet:
@@ -720,7 +729,14 @@ function main() {
     //RDebug.stopTimer(0, "initializing add-ons");
 
     // auto load scripts in AutoLoad folders for global script engine:
-    var files = RAutoLoadEcma.getAutoLoadFiles();
+
+    var files;
+    if (RSettings.isQt(6)) {
+        files = RAutoLoadJs.getAutoLoadFiles();
+    }
+    else {
+        files = RAutoLoadEcma.getAutoLoadFiles();
+    }
     for (i=0; i<files.length; i++) {
         include(files[i]);
     }
@@ -739,9 +755,9 @@ function main() {
     appWin.setCorner(Qt.TopRightCorner, Qt.RightDockWidgetArea);
     appWin.setCorner(Qt.BottomRightCorner, Qt.RightDockWidgetArea);
     appWin.setTabPosition(Qt.RightDockWidgetArea, QTabWidget.West);
-    appWin.setTabPosition(Qt.LeftDockWidgetArea, QTabWidget.South);
+    appWin.setTabPosition(Qt.LeftDockWidgetArea, QTabWidget.East);
     appWin.setTabPosition(Qt.TopDockWidgetArea, QTabWidget.South);
-    appWin.setTabPosition(Qt.BottomDockWidgetArea, QTabWidget.West);
+    appWin.setTabPosition(Qt.BottomDockWidgetArea, QTabWidget.South);
 
     setUpDragAndDrop(appWin);
 
@@ -752,13 +768,13 @@ function main() {
 
     RPluginLoader.postInitPlugins(RPluginInterface.GotMainWindowBeforeShow);
 
-    if (!QCoreApplication.arguments().contains("-no-show")) {
+    if (!args.contains("-no-show")) {
         appWin.show();
     }
 
     if (!isNull(splash)) {
         splash.close();
-        splash.destroy();
+        destr(splash);
     }
 
     postInitAddOns(addOns);
@@ -786,7 +802,7 @@ function main() {
         }
     }
 
-    // open files clicked in Finder if application is alreay running (Mac):
+    // open files clicked in Finder if application is already running (Mac):
     qApp.fileOpenRequestReceived.connect(function(fileName) {
         openFiles([fileName], false);
     });
@@ -798,7 +814,11 @@ function main() {
     var clickedFilesAndArgs = filesToOpen.concat(args.slice(1));
     QCoreApplication.processEvents();
     appWin.setProperty("starting", true);
-    openFiles(clickedFilesAndArgs, !recovered);
+    var restored = false;
+    if (typeof(restoreFiles)=="function") {
+        restored = restoreFiles();
+    }
+    openFiles(clickedFilesAndArgs, !recovered && !restored);
     appWin.setProperty("starting", false);
 
     RPluginLoader.postInitPlugins(RPluginInterface.LoadedFiles);
@@ -831,7 +851,7 @@ function main() {
     // don't use RSettings below this point
 
     // and we're done:
-    appWin.destroy();
+    destr(appWin);
     qDebug("done");
 }
 
